package edu.mermet.tp8;


import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.Locale;
import java.util.ResourceBundle;
import javax.swing.*;

import edu.mermet.tp8.fenetres.FenetreBoutons;
import edu.mermet.tp8.fenetres.FenetreConversion;
import edu.mermet.tp8.fenetres.FenetreDiaporama;
import edu.mermet.tp8.fenetres.FenetreTexte;

/**
 * @author brunomermet
 */
public class Application extends JFrame implements PropertyChangeListener
{
    private PropertyChangeSupport propertyChangeSupport;

    private JInternalFrame conversion;
    private JInternalFrame texte;
    private JInternalFrame diaporama;
    private JInternalFrame boutons;

    private Action actionAfficherConversion;
    private Action actionAfficherTexte;
    private Action actionAfficherDiaporama;
    private Action actionAfficherBoutons;

    private Action actionLangue;

    private JMenu menuFichier;
    private JMenuItem quitter;

    private JMenu menuLangue;

    private JMenu menuApplication;
    private JMenuItem itemConversion;
    private JMenuItem itemTexte;
    private JMenuItem itemDiaporama;
    private JMenuItem itemBoutons;

    private ButtonGroup groupLangue;
    private JRadioButtonMenuItem menuItemFrancais;
    private JRadioButtonMenuItem menuItemAnglais;

    private ResourceBundle bundle;

    public Application()
    {
        super("multi-fenêtres");
        init();

        this.propertyChangeSupport = new PropertyChangeSupport(this);
        addPropertyChangeListener((PropertyChangeListener)conversion);
        addPropertyChangeListener((PropertyChangeListener)texte);
        addPropertyChangeListener((PropertyChangeListener)diaporama);
        addPropertyChangeListener((PropertyChangeListener)boutons);
        addPropertyChangeListener((PropertyChangeListener)this);
        bundle = ResourceBundle.getBundle("application", new Locale("fr", "FR"));

    }

    private void init()
    {
        this.setContentPane(new JDesktopPane());

        // ****** Barre de menu ******
        JMenuBar barre = new JMenuBar();

        // ------ menu Fichier ------
        menuFichier = new JMenu("Fichier");
        menuFichier.setMnemonic(KeyEvent.VK_F);
        quitter = new JMenuItem("Quitter");
        quitter.addActionListener(ae -> System.exit(0));
        quitter.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Q, InputEvent.CTRL_DOWN_MASK));
        menuFichier.add(quitter);
        barre.add(menuFichier);
        this.setJMenuBar(barre);

        // ------ menu Applications ------
        menuApplication = new JMenu("Applications");
        menuApplication.setMnemonic(KeyEvent.VK_A);
        actionAfficherConversion = new ActionAfficherConversion();
        itemConversion = new JMenuItem(actionAfficherConversion);
        menuApplication.add(itemConversion);
        actionAfficherTexte = new ActionAfficherTexte();
        itemTexte = new JMenuItem(actionAfficherTexte);
        menuApplication.add(itemTexte);
        actionAfficherDiaporama = new ActionAfficherDiaporama();
        itemDiaporama = new JMenuItem(actionAfficherDiaporama);
        menuApplication.add(itemDiaporama);
        actionAfficherBoutons = new ActionAfficherBoutons();
        itemBoutons = new JMenuItem(actionAfficherBoutons);
        menuApplication.add(itemBoutons);
        barre.add(menuApplication);

        // ------ menu Langue ------
        menuLangue = new JMenu("Langue");
        ActionLangue fr = new ActionLangue(new Locale("fr", "FR"));
        ActionLangue en = new ActionLangue(new Locale("en", "US"));

        groupLangue = new ButtonGroup();

        menuItemFrancais = new JRadioButtonMenuItem(fr);
        menuItemFrancais.setSelected(true);
        groupLangue.add(menuItemFrancais);
        menuLangue.add(menuItemFrancais);

        menuItemAnglais = new JRadioButtonMenuItem(en);
        groupLangue.add(menuItemAnglais);
        menuLangue.add(menuItemAnglais);
        barre.add(menuLangue);
        // ****** Fin barre de menu ******


        // ****** Création des fenêtres ******

        // ------ fenêtre conversion ------
        conversion = new FenetreConversion(actionAfficherConversion);
        this.add(conversion);

        // ------ fenêtre texte ------
        texte = new FenetreTexte(actionAfficherTexte);
        this.add(texte);

        // ------ fenêtre diaporama ------
        diaporama = new FenetreDiaporama(actionAfficherDiaporama);
        this.add(diaporama);

        // ------ fenêtre boutons ------
        boutons = new FenetreBoutons(this, actionAfficherBoutons);
        this.add(boutons);
        // ****** Fin création fenêtres ******


        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        setSize(600, 300);
        this.setLocationRelativeTo(null);
        setVisible(true);
    }

    @Override
    public void propertyChange(PropertyChangeEvent evt)
    {
        menuItemFrancais.setText(bundle.getString("Francais"));
        menuItemAnglais.setText(bundle.getString("Anglais"));
        menuFichier.setText(bundle.getString("Fichier"));
        quitter.setText(bundle.getString("Quitter"));

        menuLangue.setText(bundle.getString("Langue"));

        itemConversion.setText(bundle.getString("Conversion"));
        itemTexte.setText(bundle.getString("Texte"));
        itemDiaporama.setText(bundle.getString("Diaporama"));
        itemBoutons.setText(bundle.getString("Boutons"));

    }

    private class ActionAfficherBoutons extends AbstractAction
    {
        public ActionAfficherBoutons()
        {
            super("Boutons");
            putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_B, InputEvent.CTRL_DOWN_MASK));
            putValue(Action.MNEMONIC_KEY, KeyEvent.VK_B);
        }

        @Override
        public void actionPerformed(ActionEvent ae)
        {
            boutons.setVisible(true);
            enableBoutons(false);
        }
    }

    private class ActionAfficherDiaporama extends AbstractAction
    {
        public ActionAfficherDiaporama()
        {
            super("Diaporama");
            putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_D, InputEvent.CTRL_DOWN_MASK));
            putValue(Action.MNEMONIC_KEY, KeyEvent.VK_D);
        }

        @Override
        public void actionPerformed(ActionEvent ae)
        {
            diaporama.setVisible(true);
            enableDiaporama(false);
        }
    }

    private class ActionAfficherTexte extends AbstractAction
    {
        public ActionAfficherTexte()
        {
            super("Saisie de texte");
            putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_T, InputEvent.CTRL_DOWN_MASK));
            putValue(Action.MNEMONIC_KEY, KeyEvent.VK_T);
        }

        @Override
        public void actionPerformed(ActionEvent ae)
        {
            texte.setVisible(true);
            enableTexte(false);
        }
    }

    public class ActionAfficherConversion extends AbstractAction
    {
        public ActionAfficherConversion()
        {
            super("Conversion Celsius/Farenheit");
            putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_C, InputEvent.CTRL_DOWN_MASK));
            putValue(Action.MNEMONIC_KEY, KeyEvent.VK_C);
        }

        @Override
        public void actionPerformed(ActionEvent ae)
        {
            conversion.setVisible(true);
            enableConversion(false);
        }
    }

    public class ActionLangue extends AbstractAction
    {
        private Locale langue;
        public ActionLangue(Locale langue)
        {
            super(langue.getDisplayLanguage());
            this.langue = langue;
        }

        @Override
        public void actionPerformed(ActionEvent e)
        {
            PropertyChangeEvent event = new PropertyChangeEvent(this, "langue", null, langue);
            bundle = ResourceBundle.getBundle("application", langue);
            Application.this.firePropertyChangeEvent(event);
        }
    }

    public void enableConversion(boolean b)
    {
        actionAfficherConversion.setEnabled(b);
    }

    public void enableTexte(boolean b)
    {
        actionAfficherTexte.setEnabled(b);
    }

    public void enableDiaporama(boolean b)
    {
        actionAfficherDiaporama.setEnabled(b);
    }

    public void enableBoutons(boolean b)
    {
        actionAfficherBoutons.setEnabled(b);
    }

    public Action getActionAfficherConversion()
    {
        return actionAfficherConversion;
    }

    public Action getActionAfficherTexte()
    {
        return actionAfficherTexte;
    }

    public Action getActionAfficherDiaporama()
    {
        return actionAfficherDiaporama;
    }

    public void addPropertyChangeListener(PropertyChangeListener propertyChangeListener)
    {
        this.propertyChangeSupport.addPropertyChangeListener(propertyChangeListener);
    }

    public void removePropertyChangeListener(PropertyChangeListener propertyChangeListener)
    {
        this.propertyChangeSupport.removePropertyChangeListener(propertyChangeListener);
    }

    public void firePropertyChangeEvent(PropertyChangeEvent propertyChangeEvent)
    {
        this.propertyChangeSupport.firePropertyChange(propertyChangeEvent);
    }

    public static void main(String[] args)
    {
        SwingUtilities.invokeLater(Application::new);
    }


}
